import java.util.*;
import java.io.*;

public class InvertedIndex {
	public static final String[] docs = new String[] {"doc1.txt", "doc2.txt", "doc3.txt"};
	
	private static Map<String, Set<String>> index;
	
	public static void main(String[] args) throws Throwable {
		index = new TreeMap<String, Set<String>>();
		// Let us index each of the documents
		for (String file : docs) {
			Set<String> words = wordsIn(new Scanner(new File(file)));
			// Map each word to the file they appear in
			for (String word : words) {
				if (!index.containsKey(word)) {
					index.put(word, new TreeSet<String>());
				}
				index.get(word).add(file);
			}
		}
		// Print the inverted index in column form
		for (String word : index.keySet()) {
			System.out.println(word + ": " + index.get(word));
		}
		System.out.println();
		query();
	}
	
	// This method returns a Set of strings from the given Scanner
	private static Set<String> wordsIn(Scanner data) {
		Set<String> words = new TreeSet<String>();
		while (data.hasNext()) {
			words.add(data.next());
		}
		return words;
	}
	
	// This method prompts the user for a search query and returns a set
	// of file names that contain the search terms
	private static void query() {
		Scanner in = new Scanner(System.in);
		while (true) {
			System.out.print("Query: ");
			String q = in.nextLine();
			Set<String> words = wordsIn(new Scanner(q));
			Iterator itr = words.iterator();
			Set<String> files = new TreeSet<String>(index.get(itr.next()));
			while (itr.hasNext()) {
				files.retainAll(index.get(itr.next()));
			}
			System.out.println(files);
		}
	}
}